#ifndef __GEARLIB_H__
#define __GEARLIB_H__

#include <math.h>
#include <stdbool.h>
#include <stdint.h>
#include <assert.h>
#include <stdlib.h>

typedef uint8_t u8;
typedef uint16_t u16;
typedef uint32_t u32;
typedef uint64_t u64;
typedef uintptr_t uptr;

typedef int8_t i8;
typedef int16_t i16;
typedef int32_t i32;
typedef int64_t i64;
typedef intptr_t iptr;

typedef float_t f32;
typedef double_t f64;

typedef void* (*gr_load_proc)(const char* name);

#define GRAPI extern
#define GR_DEFAULT_MAJOR 4
#define GR_DEFAULT_MINOR 6

GRAPI bool gr_state_init();
#define gr_state_assert() assert(gr_state != NULL && "State not initialized")
#define gr_state_set(prop, val) gr_state_assert(); gr_state->prop = val
#define gr_state_get(prop) ({ gr_state_assert(); gr_state->prop; })

GRAPI bool gr_window_init(i32 major, i32 minor);
GRAPI bool gr_window_create(i32 width, i32 height, const char* title);
GRAPI bool gr_window_should_close();
GRAPI void gr_window_clear(u8 red, u8 green, u8 blue);

GRAPI const char* gr_platform_get_name();
GRAPI bool gr_platform_init();
GRAPI void* gr_platform_create_window(i32 width, i32 height, const char* title);
GRAPI bool gr_platform_window_should_close();
GRAPI gr_load_proc gr_platform_get_load_proc();

GRAPI bool gr_log_add_file(char* filename);

typedef struct {
    i32 major;
    i32 minor;
    bool window_initialized;
    void* window;
} gr_state_t;

GRAPI gr_state_t* gr_state;

#endif
