#include <gearlib.h>
#include <glad/glad.h>
#include <GLFW/glfw3.h>
#include <log.c/src/log.h>

bool gr_window_init(i32 major, i32 minor) {
    if(gr_state->window_initialized == true) {
        log_error("Init already called, ignoring call to init");
        return false;
    }

    gr_state_set(major, major);
    gr_state_set(minor, minor);
    gr_state_set(window_initialized, true);

    log_info("Initialized for OpenGL %d.%d", major, minor);

    return true;
}

bool gr_window_create(i32 width, i32 height, const char* title) {
    if(!gr_state_get(window_initialized)) {
        log_info("Window was not initialized before calling create, defaulting to OpenGL %d.%d", 
                 GR_DEFAULT_MAJOR, GR_DEFAULT_MINOR);
        gr_window_init(GR_DEFAULT_MAJOR, GR_DEFAULT_MINOR);
    }

    const char* platform = gr_platform_get_name();
    if(!gr_platform_init()) {
        log_fatal("%s failed to initialize", platform);
        exit(EXIT_FAILURE);
    }

    log_info("%s initialized successfully", platform);

    void* window = gr_platform_create_window(width, height, title);
    if(window == NULL) {
        log_fatal("%s failed to create window", platform);
        exit(EXIT_FAILURE);
    }

    log_info("%s window created successfully", platform);
    log_debug("Window pointer: %p", window);

    gr_state_set(window, window);

    if(!gladLoadGLLoader(gr_platform_get_load_proc())) {
        log_fatal("Glad failed to load");
        exit(EXIT_FAILURE);
    }
    log_info("Glad loaded successfully");

    return true;
}

bool gr_window_should_close() {
    return gr_platform_window_should_close();
}

void gr_window_clear(u8 red, u8 green, u8 blue) {
    glClearColor(red / 255.0f, green / 255.0f, blue / 255.0f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
}
